#ifndef STARTUP_IMPACT_CATEGORY_CALCULATOR_H
#define STARTUP_IMPACT_CATEGORY_CALCULATOR_H

#include <FlyAdminStartUpImpact/Global>

#include <FlyAdminStartUpImpact/ProcessData>

class STARTUP_IMPACT_PARSER_EXPORT StartUpImpactCategoryCalculatorData
{
public:
    explicit StartUpImpactCategoryCalculatorData(qlonglong startMediumIOReadWriteBytes = defaultStartMediumIOReadWriteBytes,
                                                 uint startMediumCpuTimeMilliseconds = defaultStartMediumCpuTimeMilliseconds,
                                                 qlonglong startHighIOReadWriteBytes = defaultStartHighIOReadWriteBytes,
                                                 uint startHighCpuTimeMilliseconds = defaultStartHighCpuTimeMilliseconds);

    static constexpr qlonglong defaultStartMediumIOReadWriteBytes = 524288; // 512 Kilobytes
    static constexpr uint defaultStartMediumCpuTimeMilliseconds   = 250;    // 250 milliseconds

    static constexpr qlonglong defaultStartHighIOReadWriteBytes = 5242880; // 5 Megabytes
    static constexpr uint defaultStartHighCpuTimeMilliseconds   = 1500;    // 1.5 seconds

    qlonglong startMediumIOReadWriteBytes() const;
    uint startMediumCpuTimeMilliseconds() const;

    qlonglong startHighIOReadWriteBytes() const;
    uint startHighCpuTimeMilliseconds() const;

    StartUpImpactCategoryCalculatorData(const StartUpImpactCategoryCalculatorData &other);
    StartUpImpactCategoryCalculatorData &operator=(const StartUpImpactCategoryCalculatorData &other);

    StartUpImpactCategoryCalculatorData(StartUpImpactCategoryCalculatorData &&that) noexcept;
    StartUpImpactCategoryCalculatorData &operator=(StartUpImpactCategoryCalculatorData &&that) noexcept;

    inline void swap(StartUpImpactCategoryCalculatorData &lhs,
                     StartUpImpactCategoryCalculatorData &rhs) noexcept;

private:
    qlonglong m_startMediumIOReadWriteBytes;
    uint m_startMediumCpuTimeMilliseconds;

    qlonglong m_startHighIOReadWriteBytes;
    uint m_startHighCpuTimeMilliseconds;
};

class STARTUP_IMPACT_PARSER_EXPORT StartUpImpactCategoryCalculator
{
public:
    static StartUpImpact::Category standart(const StartUpImpactProcessData &processData);

    static StartUpImpact::Category fromData(const StartUpImpactProcessData &processData,
                                            const StartUpImpactCategoryCalculatorData &calcData);

    static StartUpImpact::Category fromValues(qlonglong ioReadWriteBytes,
                                              uint cpuTimeMilliseconds,
                                              const StartUpImpactCategoryCalculatorData &calcData);


private:
    StartUpImpactCategoryCalculator();
    ~StartUpImpactCategoryCalculator();
};

inline void StartUpImpactCategoryCalculatorData::swap(StartUpImpactCategoryCalculatorData &lhs,
                                                      StartUpImpactCategoryCalculatorData &rhs) noexcept
{
    std::swap(lhs.m_startMediumIOReadWriteBytes, rhs.m_startMediumIOReadWriteBytes);
    std::swap(lhs.m_startMediumCpuTimeMilliseconds, rhs.m_startMediumCpuTimeMilliseconds);
    std::swap(lhs.m_startHighIOReadWriteBytes, rhs.m_startHighIOReadWriteBytes);
    std::swap(lhs.m_startHighCpuTimeMilliseconds, rhs.m_startHighCpuTimeMilliseconds);
}

#endif // STARTUP_IMPACT_CATEGORY_CALCULATOR_H
