#ifndef STARTUP_IMPACT_PARSER_H
#define STARTUP_IMPACT_PARSER_H

#include <FlyAdminStartUpImpact/Global>

#include <FlyAdminStartUpImpact/ApplicationProperties>
#include <FlyAdminStartUpImpact/ProcessMeanMetrics>

#include <QDir>
#include <QStringList>

class STARTUP_IMPACT_PARSER_EXPORT StartUpImpactParser
{
public:
    explicit StartUpImpactParser();
    explicit StartUpImpactParser(const QString &dirPath);
    virtual ~StartUpImpactParser();

    StartUpImpactApplicationProperties loadProperties(const QString &appName) const;
    QList<StartUpImpactApplicationProperties> loadAllProperties() const;

    StartUpImpactProcessMeanMetrics loadMeanMetrics(const QString &appName) const;

    static QString standartDirPath();

    static QString analyzeInfoGroupName();
    static QString ioReadWriteBytesParamName();
    static QString cpuTimeMillisecondsParamName();
    static QString categoryParamName();

    static QString analyzeMeanMetricsGroupName();
    static QString sumIOReadWriteBytesParamName();
    static QString countIOReadWriteBytesParamName();
    static QString sumCpuTimeMillisecondsParamName();
    static QString countCpuTimeMillisecondsParamName();

    static QString fixAppName(const QString &appName);

    QStringList appNames() const;

    static StartUpImpact::Category readCategory(const QString &categoryText);
    static QString categoryText(StartUpImpact::Category category);

private:
    const QDir m_dir;

#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
    Q_DISABLE_COPY(StartUpImpactParser)
#else
    Q_DISABLE_COPY_MOVE(StartUpImpactParser)
#endif
};

#endif // STARTUP_IMPACT_PARSER_H
