#ifndef STARTUP_IMPACT_PROCESS_MEAN_METRICS_H
#define STARTUP_IMPACT_PROCESS_MEAN_METRICS_H

#include <FlyAdminStartUpImpact/Global>

class StartUpImpactProcessData;

class STARTUP_IMPACT_PARSER_EXPORT StartUpImpactProcessMeanMetrics
{
public:
    explicit StartUpImpactProcessMeanMetrics(qlonglong sumIOReadWriteBytes = 0,
                                             uint countIOReadWriteBytes = 0,
                                             uint sumCpuTimeMilliseconds = 0,
                                             uint countCpuTimeMilliseconds = 0);
    virtual ~StartUpImpactProcessMeanMetrics();

    bool isNull() const;

    qlonglong ioReadWriteBytes() const;
    uint cpuTimeMilliseconds() const;

    void addProcessData(const StartUpImpactProcessData &processData);

    void setSumIOReadWriteBytes(qlonglong sumIOReadWriteBytes);
    qlonglong sumIOReadWriteBytes() const;

    void setCountIOReadWriteBytes(uint countIOReadWriteBytes);
    uint countIOReadWriteBytes() const;

    void setSumCpuTimeMilliseconds(uint sumCpuTimeMilliseconds);
    uint sumCpuTimeMilliseconds() const;

    void setCountCpuTimeMilliseconds(uint countCpuTimeMilliseconds);
    uint countCpuTimeMilliseconds() const;

    StartUpImpactProcessMeanMetrics(const StartUpImpactProcessMeanMetrics &other);
    StartUpImpactProcessMeanMetrics &operator=(const StartUpImpactProcessMeanMetrics &other);

    StartUpImpactProcessMeanMetrics(StartUpImpactProcessMeanMetrics &&that) noexcept;
    StartUpImpactProcessMeanMetrics &operator=(StartUpImpactProcessMeanMetrics &&that) noexcept;

    inline void swap(StartUpImpactProcessMeanMetrics &lhs,
                     StartUpImpactProcessMeanMetrics &rhs) noexcept;

private:
    qlonglong m_sumIOReadWriteBytes;
    uint m_countIOReadWriteBytes;

    uint m_sumCpuTimeMilliseconds;
    uint m_countCpuTimeMilliseconds;
};

inline void StartUpImpactProcessMeanMetrics::swap(StartUpImpactProcessMeanMetrics &lhs,
                                                  StartUpImpactProcessMeanMetrics &rhs) noexcept
{
    std::swap(lhs.m_sumIOReadWriteBytes, rhs.m_sumIOReadWriteBytes);
    std::swap(lhs.m_countIOReadWriteBytes, rhs.m_countIOReadWriteBytes);
    std::swap(lhs.m_sumCpuTimeMilliseconds, rhs.m_sumCpuTimeMilliseconds);
    std::swap(lhs.m_countCpuTimeMilliseconds, rhs.m_countCpuTimeMilliseconds);
}

#endif // STARTUP_IMPACT_PROCESS_MEAN_METRICS_H
